package cleii.data;

/*
 * Lista - singolarmente concatenata
 *       - non ordinata
 *       - non identificata dal suo primo elemento
 *       - stile Object Oriented
 *       - con i GENERICS per il tipo degli elementi contenuti
 */

class Elem<T> {
	private T value;
	private Elem<T> next;
	
	public Elem (T value, Elem<T> next) {
		this.value=value;
		this.next=next;
	}
	
	public T getValue() {
		return value;
	}

	public Elem<T> getNext() {
		return next;
	}
	
	void setNext (Elem<T> next) {
		this.next=next;
	}
}


public class List <T> {
	private Elem<T> first; //riferimento al primo elemento della lista
	private int length;    //lunghezza della lista
	
	public List() {
		first=null;
		length=0;
	}
	
	public Elem<T> getFirst(){
		return first;
	}
	
	public void insert (T value) { //aggiungo l'elemento in testa
		first = new Elem<T>(value, first);
		length++;
	}
	
	public T search (T value) {
		for (Elem<T> i = first; i!=null; i = i.getNext()) {
			if (i.getValue().equals(value)) { 
				return i.getValue();
			}
		}
		return null;		
	}
	
	public int length() {
		return length;
	}
	
	/*
	 * Se trova un elemento uguale a value, lo rimuove e restituisce true
	 * Altrimenti restituisce false
	 */
	public boolean delete (T value) {
		if (value==null) return false;
		Elem<T> prev = null;
		Elem<T> e = first;
		boolean trovato=false;
		while (e!=null && !trovato) {
			if (value.equals( e.getValue() ) ) {
				trovato=true;
			} else {
				prev = e;
				e = e.getNext();
			}
		}
		if (!trovato) return false;
		//e si riferisce all'elemento che devo cancellare
		if (e==first) { //cancello il primo elemento (in testa alla lista)
			first = first.getNext();
		} else { //cancello un elemento successivo al primo 
			prev.setNext(e.getNext());
		}
		length--;
		return true;
	}
	
	
	public String toString () {// esempio: <1, 4, 5>		
		String s = "<";
		for (Elem<T> i = first; i!=null; i = i.getNext()) {
			s = s + i.getValue();
			if (i.getNext()!=null) {
				s = s + ", ";
			}
		}
		s = s + ">";
		return s;
	}
	
}
